/*
 * plugin.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"
#include <Winamp/wa_ipc.h>

#define WA_TOGGLE_VIS 40192

plugin_t *plugin_init(plugin_t *p, In_Module *mod, pb_str_t *root,
    pb_str_t *home)
{
  p->mod=mod;
  p->root=root;
  p->home=home;

  prefs_load(&p->ps.play,&p->ps.conv,home,mod->hMainWindow);

  if (NULL==decoder_init(&p->dec,&p->ps.play,mod))
    goto error;

  return p;
error:
  plugin_cleanup(p);

  return NULL;
}

plugin_t *plugin_cleanup(plugin_t *p)
{
  decoder_cleanup(&p->dec);

  if (NULL==p->home)
    free(p->home);

  if (NULL==p->root)
    free(p->root);

  return p;
}

plugin_t *plugin_start(plugin_t *p, const in_char *fn)
{
  HWND hMainWindow=p->mod->hMainWindow;
  int visrunning=SendMessage(hMainWindow,WM_WA_IPC,0,IPC_ISVISRUNNING);

  if (NULL==decoder_start(&p->dec,fn,576))
    goto error;

  if (!p->dec.is_video) {
    screen_close(&p->dec.v.s);

    if (0!=p->ps.play.general.vis&&!visrunning)
      PostMessage(hMainWindow,WM_COMMAND,WA_TOGGLE_VIS,0);
  }

  return p;
error:
  return NULL;
}

void plugin_stop(plugin_t *p, int killed)
{
  decoder_stop(&p->dec,killed?plugin_killed:NULL,p);
}

void plugin_killed(void *data)
{
  plugin_t *p=data;
  HWND hMainWindow=p->mod->hMainWindow;
  int visrunning=SendMessage(hMainWindow,WM_WA_IPC,0,IPC_ISVISRUNNING);

  if (p->dec.is_video&&!visrunning)
    screen_close(&p->dec.v.s);
}
