/*
 * plugin.h
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PLUGIN_H
#define PLUGIN_H

#include <stdio.h>
#include <windows.h>
#include <tchar.h>
#include <Winamp/in2.h>
#include <ffsox.h>
#include "vout.h"

//#define DEBUG
//#define LOCAL_AR
//#define MINGW_LINK

#if defined (DEBUG) // {
//#define DEBUG_TAGS
#endif // }

/*
 * +---------+                   +-------+
 * |         |                   |       |
 * | FFmpeg  |                   |  SoX  |
 * |         |                   |       |
 * +----+----+                   +---+---+
 *      |                            |
 *      |                            |
 * +----+----+     +-------+     +---+---+     +---------+     +-------+
 * |         |     |       |     |       |     |         |     |       |
 * |         +---->+ queue +<----+ audio +---->+ samples +<----+ main  |
 * |         |     |       |     |       |     |         |     |       |
 * |         |     +-------+     +-------+     +---------+     +-------+
 * |         |                     loop                          loop
 * | plugin  |
 * |         |     +-------+     +-------+     +---------+
 * |         |     |       |     |       |     |         |
 * |         +---->+ queue +<----+ video +---->+ screen  |
 * |         |     |       |     |       |     |         |
 * +---------+     +-------+     +-------+     +---------+
 *    loop                         loop   
 */

///////////////////////////////////////////////////////////////////////////////
#define MODULE "in_ffsox.dll"

#if ! defined (LABEL)
  #define LABEL "FFSoX Player 2"
#endif

#if ! defined (VERSION)
  #define VERSION "0.0"
#endif

#if ! defined (CAPTION)
  #define CAPTION LABEL " v" VERSION
#endif

#define ZERO(p) \
  memset(p,0,sizeof(*p))
#define Q2DB(q) \
  (20.0*log10(q))
#define DB2Q(db) \
  (pow(10.0,0.05*(db)))

#if 1 // {
#define round(x) \
  floor((x)+0.5)
#define stricmp(s1,s2) \
  _stricmp(s1,s2)
#endif // }

#if defined (DEBUG) // {
  #define DPUTS(s) fputs(s,stderr)
  #define DPRINTF(fmt,...) fprintf(stderr,fmt,__VA_ARGS__)
  #define DWPUTS(s) fwputs(s,stderr)
  #define DWPRINTF(fmt,...) fwprintf(stderr,fmt,__VA_ARGS__)
#else // } {
  #define DPUTS(s)
  #define DPRINTF(fmt,...)
  #define DWPUTS(s)
  #define DWPRINTF(fmt,...)
#endif // }

///////////////////////////////////////////////////////////////////////////////
int ffsox_load_ffmpeg(pb_str_t *home);
int ffsox_load_sox(pb_str_t *home);
int ffsox_load_dlgitemurl(pb_str_t *home);

///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam,
    LPARAM lParam);

///////////////////////////////////////////////////////////////////////////////
typedef enum state {
  STATE_RUN=0,
  STATE_PAUSE,
  STATE_EOF,
  STATE_KILL
} state_t;

typedef struct ap {
  int rate;
  int nch;
  int bps;
} ap_t;

typedef struct decoder decoder_t;
typedef struct plugin plugin_t;
#if defined (LOCAL_AR) // {
typedef struct ar ar_t;
#endif // }

///////////////////////////////////////////////////////////////////////////////
typedef struct buf {
  char *rep;
  char *wp;
  char *mp;
} buf_t;

buf_t *buf_init(buf_t *buf, char *rep, size_t size);
buf_t *buf_cleanup(buf_t *buf);

buf_t *buf_realloc(buf_t *buf, size_t n);
buf_t *buf_append_n(buf_t *buf, char *cs, size_t n);
buf_t *buf_append(buf_t *buf, char *cs);

///////////////////////////////////////////////////////////////////////////////
typedef struct wbuf {
  wchar_t *rep;
  wchar_t *wp;
  wchar_t *mp;
} wbuf_t;

wbuf_t *wbuf_init(wbuf_t *wb, wchar_t *rep, size_t n);

wbuf_t *wbuf_add(wbuf_t *wb, wchar_t *wcs);
wbuf_t *wbuf_add_wva(wbuf_t *wb, wchar_t *fmt, ...);
wbuf_t *wbuf_add_va(wbuf_t *wb, char *fmt, ...);

///////////////////////////////////////////////////////////////////////////////
#define SOX_DITHER_OFF "off"
#define SOX_DITHER_FILTER "-f "

typedef struct prefs {
  pb_str_t *path;

  struct {
    buf_t display;
    buf_t plugin;
  } exts;

  struct {
    int bps;
    int vis;
    int vid;
  } general;

  struct {
#if ! defined (RG_WEIGHT_BOX) // {
	int off;
#endif // }
    double weight;
    int comment;
  } rg;

  struct {
    int queue_size;
    double preamp;
  } audio;

  struct {
    char mode[16];
    int rate;
    int rate_min;
    int rate_mul;
    char rate_q[16];
    char dither[32];
  } sox;

  struct {
    int queue_size;
    double min_drift;
    double max_drift;
  } video;
} prefs_t;

void prefs_load(prefs_t *play, prefs_t *conv, pb_str_t *home,
    HWND hMainWindow);
void prefs_edit(prefs_t *play, prefs_t *conv, HWND hParent);

#define ps_sox_mode(ps,val) \
  (0==strcmp(#val,(ps)->sox.mode))
#define ps_sox_mode_off(ps) \
  ps_sox_mode(ps,off)
#define prefs_sox_mode_auto(ps) \
  ps_sox_mode(ps,auto)
#define ps_sox_mode_on(ps) \
  ps_sox_mode(ps,on)

///////////////////////////////////////////////////////////////////////////////
typedef struct track {
  struct {
#define INFO_BEGIN(id) \
    pb_str_t *id;
#define INFO_ALT(id,id2)
#define INFO_END()
#include "info.h"
#undef INFO_END
#undef INFO_ALT
#undef INFO_BEGIN
  } info;

  pb_str_t *fn;
  prefs_t *ps;
  ffi_t ffi;
} track_t;

track_t *track_init(track_t *t, const in_char *fn, prefs_t *ps, int full);
void track_cleanup(track_t *t);

track_t *track_reopen(track_t *t, const in_char *fn, prefs_t *ps, int full);
void track_getfileinfo(track_t *t, in_char *title, int *length_in_ms);
in_char *track_getfileinfo_ext(track_t *t, char *metadata, wchar_t *ret,
    int retlen);

int track_sox_needed(track_t *t, ap_t *in, ap_t *out);

char *track_get_tag(track_t *t, char *key);
double track_get_q(track_t *t);
int track_get_length(track_t *t);
in_char *track_get_overwrite_comment(track_t *t, in_char *buf, size_t n);
in_char *track_get_formatinformation(track_t *t, in_char *buf, size_t n);

#define INFO_BEGIN(id) \
in_char *track_get_##id(track_t *t, in_char *buf, size_t n);
#define INFO_ALT(id,id2)
#define INFO_END()
#include "info.h"
#undef INFO_END
#undef INFO_ALT
#undef INFO_BEGIN

///////////////////////////////////////////////////////////////////////////////
typedef struct timer {
  volatile state_t state;
  HANDLE hMutex;
  HANDLE hEvent;
  HANDLE hTimerQueue;
} timer_t;

timer_t *timer_init(timer_t *t);
void timer_cleanup(timer_t *t);

void timer_kill(timer_t *t);
void timer_sleep(timer_t *t, DWORD msInterval);

///////////////////////////////////////////////////////////////////////////////
typedef struct node {
  struct node *next;
  AVPacket pkt;
} node_t;

///////////////////////////////////////////////////////////////////////////////
typedef struct list {
  node_t *head;
  node_t *tail;
} list_t;

node_t *list_write(list_t *l, node_t *n);
node_t *list_read(list_t *l);

///////////////////////////////////////////////////////////////////////////////
#define QUEUE_SIZE 4

#define queue_overflow(q) \
  ((q)->size<=(q)->count)

typedef struct queue {
  decoder_t *dec;
  char *name;
  HANDLE hMutex;
  HANDLE hEventWrite;
  HANDLE hEventRead;

  int size;
  volatile int count;

  list_t active;
#if defined (QUEUE_FREE) // {
  list_t free;
#else // } {
  list_t *free;
#endif // }
} queue_t;

queue_t *queue_init(queue_t *q, decoder_t *dec, char *name, int size);
queue_t *queue_cleanup(queue_t *q);

void queue_reset(queue_t *q, int size);
void queue_flush(queue_t *q);

// non-blocking.
int queue_write(queue_t *q, AVPacket *pkt);
// blocking.
int queue_read(queue_t *q, AVPacket *pkt);

///////////////////////////////////////////////////////////////////////////////
typedef struct audio audio_t;

typedef struct samples {
  HANDLE hMutex;
  HANDLE hEventWrite;
  HANDLE hEventRead;

  volatile state_t state;
  volatile double time;
  track_t *t;
  int bps;
  double q;

  size_t size;

  struct {
    char *rep;
    volatile char *wp;
    char *mp;
  } in;

  struct {
    char *rep;
  } out;
} samples_t;

samples_t *samples_init(samples_t *s);
samples_t *samples_cleanup(samples_t *s);

void samples_kill(samples_t *s);
void samples_clear(samples_t *s);
samples_t *samples_realloc(samples_t *s, track_t *t, size_t nsam,
    int nch, int bps);

void samples_wait_lock(samples_t *s);
void samples_notify_unlock(samples_t *s);

#if defined (LOCAL_AR) // {
void samples_copy(samples_t *s, ar_t *ar);
#else // } {
void samples_copy(samples_t *s, ffsox_ar_t *ar);
#endif // }

///////////////////////////////////////////////////////////////////////////////
typedef struct screen {
  In_Module *mod;
  vout_t *vo;
  
  int open;
  int width;
  int height;
  double ar;
} screen_t;

screen_t *screen_init(screen_t *s, In_Module *mod);
screen_t *screen_cleanup(screen_t *s);

void screen_open(screen_t *s, int width, int height, double ar);
void screen_draw(screen_t *s, AVPicture *pict);
void screen_close(screen_t *s);
void screen_done(screen_t *s);

#if defined (LOCAL_AR) // {
///////////////////////////////////////////////////////////////////////////////
#define ar_valid(ar) \
  (AR_STATE_READ==(ar)->state||AR_STATE_FLUSH==(ar)->state)

enum {
  AR_STATE_READ,
  AR_STATE_FLUSH,
  AR_STATE_DONE,
  AR_STATE_KILL,
  AR_STATE_ERROR,
};

struct ar {
  int state;
  audio_t *a;
  queue_t *q;
  plugin_t *p;
  AVCodecContext *dc;
  int touched;
  AVFrame *frame;
  AVPacket pkt;
};

void ar_init(ar_t *ar, audio_t *a);
void ar_start(ar_t *ar);
void ar_next(ar_t *ar);
#endif // }

///////////////////////////////////////////////////////////////////////////////
typedef struct audio {
  decoder_t *dec;
#if defined (LOCAL_AR) // {
  AVFrame *frame;
#endif // }
  queue_t q;
  samples_t s;

#if defined (LOCAL_AR) // {
  ar_t ar;
#else // } {
  ffsox_ar_t ar;
#endif // }
  sox_effects_chain_t *chain;
  ffi_st_t *st;

  ap_t in;
  ap_t out;

  HANDLE hThread;
} audio_t;

audio_t *audio_init(audio_t *a, decoder_t *dec);
audio_t *audio_cleanup(audio_t *a);

audio_t *audio_realloc(audio_t *a, track_t *t, size_t nsam);
void audio_reset(audio_t *a);
audio_t *audio_run(audio_t *a);
void audio_wait(audio_t *a);
void audio_stop(audio_t *a);
void audio_kill(audio_t *a);

DWORD WINAPI audio_thread(LPVOID lpParam);

///////////////////////////////////////////////////////////////////////////////
#define VIDEO_MIN_DRIFT (-200.0)
#define VIDEO_MAX_DRIFT (8000.0)

typedef struct video {
  decoder_t *dec;
  In_Module *mod;
  AVFrame *frame;
  timer_t timer;
  queue_t q;
  screen_t s;

  ffi_st_t *st;
  double time_base;
  AVPicture pict;
  HANDLE hThread;

  int crop;
  int width;
  int height;
  int x;
  int y;
  int64_t next_pts;
} video_t;

video_t *video_init(video_t *v, decoder_t *dec, In_Module *mod);
video_t *video_cleanup(video_t *v);

video_t *video_realloc(video_t *v, track_t *t);
void video_reset(video_t *v);
video_t *video_run(video_t *v);
void video_wait(video_t *v);
void video_stop(video_t *v);
void video_kill(video_t *v);

DWORD WINAPI video_thread(LPVOID lpParam);

///////////////////////////////////////////////////////////////////////////////
struct decoder {
  prefs_t *ps;
  volatile state_t state;
  HANDLE hMutex;
  HANDLE hEventWrite;
#if ! defined (QUEUE_FREE) // {
  list_t free;
#endif // }
  audio_t a;
  video_t v;

#if defined (PLAY) // {
  track_t *t;
#else // } {
  track_t t;
#endif // }
  int is_video;
  HANDLE hThread;
};

decoder_t *decoder_init(decoder_t *dec, prefs_t *ps, In_Module *mod);
decoder_t *decoder_cleanup(decoder_t *dec);

double decoder_get_time(decoder_t *dec, In_Module *mod);
#if defined (PLAY) // {
decoder_t *decoder_start(decoder_t *dec, track_t *t, const in_char *fn);
#else // } {
decoder_t *decoder_start(decoder_t *dec, const in_char *fn, size_t nsam);
#endif // }
void decoder_stop(decoder_t *dec, void (*killed)(void *), void *data);
void decoder_reset(decoder_t *dec);
decoder_t *decoder_run(decoder_t *dec);
void decoder_pause(decoder_t *dec);
void decoder_unpause(decoder_t *dec);
void decoder_wait(decoder_t *dec);
void decoder_kill(decoder_t *dec);
void decoder_eof(decoder_t *dec);
int decoder_seek(decoder_t *dec, int64_t pos_ms);
void decoder_write(decoder_t *dec, queue_t *q, AVPacket *pkt);

DWORD WINAPI decoder_thread(LPVOID lpParam);

///////////////////////////////////////////////////////////////////////////////
struct plugin {
  In_Module *mod;
  pb_str_t *root;
  pb_str_t *home;

  struct {
    prefs_t play;
    prefs_t conv;
  } ps;

  track_t list;
#if defined (PLAY) // {
  track_t play;
#endif // }

  decoder_t dec;
};

#define plugin_run(p) \
  (NULL==decoder_run(&(p)->dec)?NULL:p)
#define plugin_pause(p) \
  (decoder_pause(&(p)->dec))
#define plugin_unpause(p) \
  (decoder_unpause(&(p)->dec))
#define plugin_wait(p) \
  (decoder_wait(&(p)->dec))
#define plugin_kill(p) \
  (decoder_kill(&(p)->dec))
#define plugin_seek(p,pos_ms) \
  (decoder_seek(&(p)->dec,pos_ms))

plugin_t *plugin_init(plugin_t *p, In_Module *mod, pb_str_t *root,
    pb_str_t *home);
plugin_t *plugin_cleanup(plugin_t *p);

plugin_t *plugin_start(plugin_t *p, const in_char *fn);
void plugin_stop(plugin_t *p, int killed);
void plugin_killed(void *data);

///////////////////////////////////////////////////////////////////////////////
sox_effects_chain_t *ffsox_chain(audio_t *a);
sox_effect_handler_t const *ffsox_reader();
sox_effect_handler_t const *ffsox_writer();

///////////////////////////////////////////////////////////////////////////////

#endif // PLUGIN_H
