/*
 * prefs.h
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PREFS_H
#define PREFS_H

#include "plugin.h"

///////////////////////////////////////////////////////////////////////////////
typedef struct prefdef prefdef_t;
typedef struct prefgui prefgui_t;
typedef struct prefvmt prefvmt_t;

typedef struct slider slider_t;
typedef struct combo combo_t;

///////////////////////////////////////////////////////////////////////////////
enum {
  PREF_NONE=0,
  PREF_INT,
  PREF_DOUBLE,
  PREF_STRING
};

struct slider {
  union {
    struct {
      int min;
      int max;
    } i;

    struct {
      double min;
      double max;
      double step;
    } x;
  };
};

struct combo {
  char *label;

  union {
    int i;
    double x;
    char *s;
  };
};

struct prefdef {
  char *group;
  char *name;
  int offs;
  int size;
  int type;
  int id;
  prefgui_t *(*init)(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg);

  union {
    slider_t *slider;
    combo_t *combo;
  };
};

#define pref_int(pd,ps) \
  (*(int *)((char *)(ps)+(pd)->offs))
#define pref_double(pd,ps) \
  (*(double *)((char *)(ps)+(pd)->offs))
#define pref_string(pd,ps) \
  ((char *)(ps)+(pd)->offs)

///////////////////////////////////////////////////////////////////////////////
void prefs_read(prefs_t *ps, prefdef_t *pd, prefs_t *dps);
void prefs_read_play(prefs_t *ps, prefdef_t *pd);
void prefs_read_conv(prefs_t *ps, prefdef_t *pd);
void prefs_write(prefs_t *ps, prefdef_t *pd);

void prefs_general_load(prefs_t *ps);
void prefs_audio_load(prefs_t *ps);
void prefs_video_load(prefs_t *ps);
//void prefs_conv_load(prefs_t *ps);
void prefs_conv_general_load(prefs_t *ps);
void prefs_conv_audio_load(prefs_t *ps);
void prefs_help_load(prefs_t *ps);

///////////////////////////////////////////////////////////////////////////////
struct prefvmt {
  void (*set)(prefgui_t *pg, prefs_t *ps, HWND hDlg);
  void (*get)(prefgui_t *pg, prefs_t *ps, HWND hDlg);
  int (*get_int)(prefgui_t *pg, HWND hDlg);
  double (*get_double)(prefgui_t *pg, HWND hDlg);
};

struct prefgui {
  prefvmt_t *vmt;
  prefdef_t *pd;
  int id;
};

prefgui_t *prefs_check(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg);
prefgui_t *prefs_slider(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg);
prefgui_t *prefs_combo(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg);

///////////////////////////////////////////////////////////////////////////////
// tab control page definition
typedef struct tcp {
  wchar_t *title;
  int id;
  DLGPROC proc;
  size_t offs;
} tcp_t;

// tab control
typedef struct tabs {
  int id;
  HWND *hPages;
  HWND hCurPage;
} tabs_t;

tabs_t *tabs_init(tabs_t *tc, HWND hDlg, int id, int n, tcp_t *tcp,
    HWND *hPages, char *page);
void tabs_set_page(tabs_t *tc, HWND hDlg, int i);
void tabs_change(tabs_t *tc, HWND hDlg);

///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK GeneralDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
BOOL CALLBACK AudioDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
BOOL CALLBACK VideoDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
BOOL CALLBACK HelpDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

BOOL CALLBACK ConvDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
BOOL CALLBACK ConvGeneralDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
BOOL CALLBACK ConvAudioDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

#endif // PREFS_H
