/*
 * pref_combo.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "prefs.h"
#include <commctrl.h>

static prefvmt_t vmt;

prefgui_t *prefs_combo(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg)
{
  pg->vmt=&vmt;
  pg->pd=pd;
  pg->id=pd->id;
  combo_t *c;

  for (c=pd->combo;NULL!=c->label;++c)
    SendDlgItemMessageA(hDlg,pg->id,CB_ADDSTRING,0,(LPARAM)c->label);

  vmt.set(pg,ps,hDlg);

  return pg;
}

static void set(prefgui_t *pg, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  combo_t *c;

  for (c=pd->combo;NULL!=c->label;++c) {
    switch (pd->type) {
    case PREF_INT:
      if (c->i!=pref_int(pd,ps))
        continue;

      SendDlgItemMessageA(hDlg,pg->id,CB_SETCURSEL,c-pd->combo,0);
      return;
    case PREF_DOUBLE:
      if (c->x!=pref_double(pd,ps))
        continue;

      SendDlgItemMessageA(hDlg,pg->id,CB_SETCURSEL,c-pd->combo,0);
      return;
    case PREF_STRING:
      if (0!=strcmp(c->s,pref_string(pd,ps)))
        continue;

      SendDlgItemMessageA(hDlg,pg->id,CB_SETCURSEL,c-pd->combo,0);
      return;
    default:
      break;
    }
  }

  SendDlgItemMessageA(hDlg,pg->id,CB_SETCURSEL,0,0);
}

static void get(prefgui_t *pg, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  combo_t *c=pd->combo+SendDlgItemMessage(hDlg,pg->id,CB_GETCURSEL,0,0);

  switch (pd->type) {
  case PREF_INT:
	pref_int(pd,ps)=c->i;
    break;
  case PREF_DOUBLE:
	pref_double(pd,ps)=c->x;
    break;
  case PREF_STRING:
	strncpy(pref_string(pd,ps),c->s,pd->size-1);
	pref_string(pd,ps)[pd->size-1]='\0';
    break;
  default:
    break;
  }
}

static int get_int(prefgui_t *pg, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  combo_t *c=pd->combo+SendDlgItemMessage(hDlg,pg->id,CB_GETCURSEL,0,0);

  return PREF_INT==pd->type?c->i:0;
}

static double get_double(prefgui_t *pg, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  combo_t *c=pd->combo+SendDlgItemMessage(hDlg,pg->id,CB_GETCURSEL,0,0);

  return PREF_DOUBLE==pd->type?c->x:0.0;
}

static prefvmt_t vmt={
  .set=set,
  .get=get,
  .get_int=get_int,
  .get_double=get_double
};
