/*
 * prefs_conv.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <Winamp/wa_ipc.h>
#include "prefdef.h"

///////////////////////////////////////////////////////////////////////////////
#define DEFAULT_PAGE 1

typedef prefs_conv_t priv_t;

static prefs_t conv_dps={
  // general
  .general.bps=16,
  .general.vis=0,
  .general.vid=0,

  // rg
  .rg.weight=0.0,
  .rg.comment=1,

  // audio
  .audio.queue_size=QUEUE_SIZE,
  .audio.preamp=0.0,

  // sox
  .sox.mode="auto",
  .sox.rate=44100,
  .sox.rate_min=0,
  .sox.rate_mul=0,
  .sox.rate_q="-v",
  .sox.dither="-s",

  // video
};

static tcp_t tcp[]=PREFS_CONV_TCP();

///////////////////////////////////////////////////////////////////////////////
void prefs_read_conv(prefs_t *ps, prefdef_t *pd)
{
  prefs_read(ps,pd,&conv_dps);
}

///////////////////////////////////////////////////////////////////////////////
static void ConvDlgDispatch(priv_t *p, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  int i;

  for (i=0;i<PREFS_CONV_NPAGES;++i)
    SendMessage(p->hPages[i],uMsg,wParam,lParam);
}

BOOL CALLBACK ConvDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  priv_t *p=WM_INITDIALOG==uMsg?(void *)lParam
      :(void *)GetWindowLongPtr(hDlg,GWLP_USERDATA);
  int n;
  char buf[32];

  switch(uMsg) {
  case WM_INITDIALOG:
    SetWindowLongPtr(hDlg,GWLP_USERDATA,lParam);
    n=GetPrivateProfileIntA(CONV_GENERAL,"page",DEFAULT_PAGE,
	    PS_UTF8(p->ps->path));
    tabs_init(&p->tc,hDlg,IDC_TAB_PREFS_CONV,n-1,tcp,p->hPages,
	    (char *)&p->page);
    return TRUE;
  case WM_NOTIFY:
    if (IDC_TAB_PREFS_CONV==wParam&&TCN_SELCHANGE==((NMHDR *)lParam)->code) {
      tabs_change(&p->tc,hDlg);
      return TRUE;
    }

    break;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      n=TabCtrl_GetCurSel(GetDlgItem(hDlg,p->tc.id)); 
	  sprintf(buf,"%d",1+n);
      WritePrivateProfileStringA(CONV_GENERAL,"page",buf,PS_UTF8(p->ps->path));
	  ConvDlgDispatch(p,WM_COMMAND,IDOK,0);
      return TRUE;
    case IDC_BUTTON_DEFAULT:
	  ConvDlgDispatch(p,WM_COMMAND,IDC_BUTTON_DEFAULT,(LPARAM)&conv_dps);
      tabs_set_page(&p->tc,hDlg,DEFAULT_PAGE-1);
      return TRUE;
	default:
	  break;
    }

    break;
  default:
    break;
  }

  return FALSE;
}
