/*
 * wbuf.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdarg.h>
#include "plugin.h"

wbuf_t *wbuf_init(wbuf_t *wb, wchar_t *rep, size_t n)
{
  wb->rep=rep;
  wb->mp=rep+n;
  wb->wp=rep;
  wb->wp[0]=L'\0';

  return wb;
}

wbuf_t *wbuf_add(wbuf_t *wb, wchar_t *wcs)
{
  size_t n=wb->mp-wb->wp-1;

  if (0<n) {
    wcsncpy(wb->wp,wcs,n);
    wb->wp[n]=L'\0';
    wb->wp+=wcslen(wb->wp);
  }

  return wb;
}

wbuf_t *wbuf_add_wva(wbuf_t *wb, wchar_t *fmt, ...)
{
  size_t n=wb->mp-wb->wp-1;
  va_list ap;

  if (0<n) {
    va_start(ap,fmt);
    wb->wp+=_vsnwprintf(wb->wp,n,fmt,ap);
    wb->wp[0]=L'\0';
    va_end(ap);
  }

  return wb;
}

wbuf_t *wbuf_add_va(wbuf_t *wb, char *fmt, ...)
{
  enum { N=128 };
  char buf[N];
  size_t n;
  va_list ap;

  va_start(ap,fmt);
  n=_vsnprintf(buf,N-1,fmt,ap);
  buf[n]=L'\0';
  va_end(ap);

  n=wb->mp-wb->wp-1;

  n=MultiByteToWideChar(
    CP_UTF8,    // __in       UINT CodePage,
    0,          // __in       DWORD dwFlags,
    buf,        // __in       LPCSTR lpMultiByteStr,
    -1,         // __in       int cbMultiByte,
    wb->wp,     // __out_opt  LPWSTR lpWideCharStr,
    n           // __in       int cchWideChar
  );

  wb->wp+=n-1;

  return wb;
}
